require "sourcify"

module CodeAssertions
    CodeAssertionFailed = Class.new(Exception)
    @@assertions_check  = true
    
    def assertions_off
        @@assertions_check = false
    end
    
    alias :do_not_assert :assertions_off
    
    def assert(message=nil, &block)
        return unless @@assertions_check
        unless block
            warn "Empty assertion passed."
            return
        end
        
        begin
            message = "Assertion failed: " + block.to_source[6...-1].strip unless message
        rescue Exception
            message = "Assertion failed: <no message provided>"
        end
        raise CodeAssertionFailed, message unless block.call
    end
    
    def soft_assert(message=nil, &block)
        return unless @@assertions_check
        unless block
            warn "Empty assertion passed."
            return
        end
        
        begin
            message = "Assertion failed: " + block.to_source[6...-1].strip unless message
        rescue Exception
            message = "Assertion failed: <no message provided>"
        end
        warn message unless block.call
    end
end

class Object
    def assert_not_nil!
        assert("The result cannot be `nil` or `false`.") { self }
        
        self
    end
    alias :assert_nn! :assert_not_nil!
    
    def assert_type!(type)
        assert("`type` should be an instance of `Class`") { type.is_a?(Class) }
        assert("Wrong type: expected #{type.name}, given #{self.class.name}") { self.is_a?(type) }
        
        self
    end
    
    alias :at!   :assert_type!
    alias :is_a! :assert_type!
end

class File
    def assert_exist!
        assert("The specified file should exist: `#{self.path}`.") { FileTest.exist?(self.path) }
        
        self
    end
    
    def assert_not_exist!(filename)
        assert("The specified file should not exist: `#{self.path}`.") { !FileTest.exist?(self.path) }
        
        self
    end
end

class String
    def assert_empty!
        assert("Expected empty string, `#{self}` instead") { self.size == 0 }
        
        self
    end
    
    def assert_not_empty!
        assert("Expected empty string, `#{self}` instead") { self.size > 0 }
        
        self
    end
end

include CodeAssertions
